@@ -12,4 +12,4 @@
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
 
-from crystalparser.parser import CrystalParser
+from crystalparser.crystal_parser import CrystalParser
