@@ -2,62 +2,91 @@ import numpy as np
 import os
 import logging
 from ase import data as asedata
+import pint
 
-from .LammpsCommon import converter
+from .metainfo import m_env
+from nomad.parsing.parser import FairdiParser
 
-from nomad.parsing.text_parser import Quantity, UnstructuredTextFileParser
+from nomad.parsing.file_parser import Quantity, UnstructuredTextFileParser
 from nomad.datamodel.metainfo.public import section_run, section_sampling_method,\
     section_system, section_single_configuration_calculation, section_energy_contribution,\
     Workflow, MolecularDynamics
 from nomad.datamodel.metainfo.common import section_topology, section_interaction
+from .metainfo.lammps import x_lammps_section_input_output_files, x_lammps_section_control_parameters
 
 
-class Parser:
-    def __init__(self, mainfile, logger):
-        self._mainfile = mainfile
-        self._results = None
-        self._quantities = None
-        self.logger = logger
+def get_unit(units_type, property_type=None, dimension=3):
+    mole = 6.022140857e+23
 
-    def __getitem__(self, key):
-        val = self.results.get(key, None)
-        return val
+    units_type = units_type.lower()
+    if units_type == 'real':
+        units = dict(
+            mass=pint.Quantity(1 / mole, 'g'), distance='angstrom', time='fs',
+            energy=pint.Quantity(1 / mole, 'kcal'), velocity='angstrom/fs',
+            force=pint.Quantity(1 / mole, 'kcal/angstrom'), torque=pint.Quantity(1 / mole, 'kcal'),
+            temperature='K', pressure='atm', dynamic_viscosity='poise', charge='elementary_charge',
+            dipole='elementary_charge*angstrom', electric_field='V/angstrom',
+            density='g/cm^%d' % dimension)
 
-    def get(self, key, default=None):
-        val = self[key]
-        if not val:
-            return default
-        return val
+    elif units_type == 'metal':
+        units = dict(
+            mass=pint.Quantity(1 / mole, 'g'), distance='angstrom', time='ps',
+            energy='eV', velocity='angstrom/ps', force='eV/angstrom', torque='eV',
+            temperature='K', pressure='bar', dynamic_viscosity='poise', charge='elementary_charge',
+            dipole='elementary_charge*angstrom', electric_field='V/angstrom',
+            density='g/cm^%d' % dimension)
 
-    @property
-    def maindir(self):
-        return os.path.dirname(self._mainfile)
+    elif units_type == 'si':
+        units = dict(
+            mass='kg', distance='m', time='s', energy='J', velocity='m/s', force='N',
+            torque='N*m', temperature='K', pressure='Pa', dynamic_viscosity='Pa*s',
+            charge='C', dipole='C*m', electric_field='V/m', density='kg/m^%d' % dimension)
 
-    @property
-    def mainfile(self):
-        return self._mainfile
+    elif units_type == 'cgs':
+        units = dict(
+            mass='g', distance='cm', time='s', energy='erg', velocity='cm/s', force='dyne',
+            torque='dyne*cm', temperature='K', pressure='dyne/cm^2', dynamic_viscosity='poise',
+            charge='esu', dipole='esu*cm', electric_field='dyne/esu',
+            density='g/cm^%d' % dimension)
 
-    @mainfile.setter
-    def mainfile(self, value):
-        self._mainfile = value
-        self._results = None
+    elif units_type == 'electron':
+        units = dict(
+            mass='amu', distance='bohr', time='fs', energy='hartree',
+            velocity='bohr/atomic_unit_of_time', force='hartree/bohr', temperature='K',
+            pressure='Pa', charge='elementary_charge', dipole='debye', electric_field='V/cm')
 
-    @property
-    def results(self):
-        if self._results is None:
-            self.parse()
-        if self._results is None:
-            self._results = {}
+    elif units_type == 'micro':
+        units = dict(
+            mass='pg', distance='microm', time='micros', energy='pg*microm^2/micros^2',
+            velocity='microm/micros', force='pg*microm/micros^2', torque='pg*microm^2/micros^2',
+            temperature='K', pressure='pg/(microm*micros^2)', dynamic_viscosity='pg/(microm*micros)',
+            charge='pC', dipole='pC*microm', electric_field='V/microm',
+            density='pg/microm^%d' % dimension)
 
-        return self._results
+    elif units_type == 'nano':
+        units = dict(
+            mass='ag', distance='nm', time='ns', energy='ag*nm^2/ns^2', velocity='nm/ns',
+            force='ag*nm/ns^2', torque='ag*nm^2/ns^2', temperature='K', pressure='ag/(nm*ns^2)',
+            dynamic_viscosity='ag/(nm*ns)', charge='elementary_charge',
+            dipole='elementary_charge*nm', electric_field='V/nm', density='ag/nm^%d' % dimension)
 
-    def parse(self):
-        pass
+    else:
+        units = dict()
+
+    if property_type:
+        unit = units.get(property_type, None)
+        if isinstance(unit, str):
+            unit = pint.Quantity(1, unit)
+        return unit
+    else:
+        for key, val in units.items():
+            if isinstance(val, str):
+                units[key] = pint.Quantity(1, val)
+        return units
 
 
-class DataParser(Parser):
-    def __init__(self, mainfile, logger):
-        super().__init__(mainfile, logger)
+class DataParser(UnstructuredTextFileParser):
+    def __init__(self):
         self._headers = [
             'atoms', 'bonds', 'angles', 'dihedrals', 'impropers', 'atom types', 'bond types',
             'angle types', 'dihedral types', 'improper types', 'extra bond per atom',
@@ -75,50 +104,40 @@ class DataParser(Parser):
             'BondBond13 Coeffs', 'AngleAngle Coeffs']
         self._interactions = [
             section for section in self._sections if section.endswith('Coeffs')]
+        super().__init__(None)
 
-    def parse(self):
-        if self.mainfile is None:
-            return
+    def init_quantities(self):
+        self._quantities = []
+        for header in self._headers:
+            self._quantities.append(
+                Quantity(header, r'\s*([\+\-eE\d\. ]+)\s*%s\s*\n' % header, comment='#'))
 
-        if not os.path.isfile(self.mainfile):
-            return
+        def get_section_value(val):
+            val = val.split('\n')
+            name = None
 
-        if self._quantities is None:
-            self._quantities = []
-            for header in self._headers:
-                self._quantities.append(
-                    Quantity(header, r'\s*([\+\-eE\d\. ]+)\s*%s\s*\n' % header, comment='#'))
+            if val[0][0] == '#':
+                name = val[0][1:].strip()
+                val = val[1:]
 
-            def get_section_value(val):
-                val = val.split('\n')
-                name = None
+            value = []
+            for i in range(len(val)):
+                v = val[i].split('#')[0].split()
+                if not v:
+                    continue
 
-                if val[0][0] == '#':
-                    name = val[0][1:].strip()
-                    val = val[1:]
+                try:
+                    value.append(np.array(v, dtype=float))
+                except Exception:
+                    break
 
-                value = []
-                for i in range(len(val)):
-                    v = val[i].split('#')[0].split()
-                    if not v:
-                        continue
+            return name, np.array(value)
 
-                    try:
-                        value.append(np.array(v, dtype=float))
-                    except Exception:
-                        break
-
-                return name, np.array(value)
-
-            for section in self._sections:
-                self._quantities.append(
-                    Quantity(
-                        section, r'\s*%s\s*(#*\s*[\s\S]*?\n)\n*([\deE\-\+\.\s]+)\n' % section,
-                        str_operation=get_section_value))
-
-        parser = UnstructuredTextFileParser(self.mainfile, self._quantities)
-
-        self._results = {key: parser[key] for key in parser.keys() if parser[key]}
+        for section in self._sections:
+            self._quantities.append(
+                Quantity(
+                    section, r'\s*%s\s*(#*\s*[\s\S]*?\n)\n*([\deE\-\+\.\s]+)\n' % section,
+                    str_operation=get_section_value))
 
     def get_interactions(self):
         styles_coeffs = []
@@ -134,61 +153,49 @@ class DataParser(Parser):
         return styles_coeffs
 
 
-class TrajParser(Parser):
-    def __init__(self, mainfile, logger):
-        super().__init__(mainfile, logger)
+class TrajParser(UnstructuredTextFileParser):
+    def __init__(self):
         self._masses = None
         self._reference_masses = dict(
             masses=np.array(asedata.atomic_masses), symbols=asedata.chemical_symbols)
         self._chemical_symbols = None
+        self._units = None
+        super().__init__(None)
 
-    def parse(self):
-        if self.mainfile is None:
-            return
+    def init_quantities(self):
 
-        if not os.path.isfile(self.mainfile):
-            return
+        def get_pbc_cell(val):
+            val = val.split()
+            pbc = [v == 'pp' for v in val[:3]]
 
-        if self._quantities is None:
-            def get_pbc_cell(val):
-                val = val.split()
-                pbc = [v == 'pp' for v in val[:3]]
+            cell = np.zeros((3, 3))
+            for i in range(3):
+                cell[i][i] = float(val[i * 2 + 4]) - float(val[i * 2 + 3])
 
-                cell = np.zeros((3, 3))
-                for i in range(3):
-                    cell[i][i] = float(val[i * 2 + 4]) - float(val[i * 2 + 3])
+            return pbc, cell
 
-                return pbc, cell
+        def get_atoms_info(val):
+            val = val.split('\n')
+            keys = val[0].split()
+            values = np.array([v.split() for v in val[1:] if v], dtype=float)
+            values = values[values[:, 0].argsort()].T
+            return {keys[i]: values[i] for i in range(len(keys))}
 
-            def get_atoms_info(val):
-                val = val.split('\n')
-                keys = val[0].split()
-                values = np.array([v.split() for v in val[1:] if v], dtype=float)
-                values = values[values[:, 0].argsort()].T
-                return {keys[i]: values[i] for i in range(len(keys))}
-
-            self._quantities = [
-                Quantity(
-                    'time_step', r'\s*ITEM:\s*TIMESTEP\s*\n\s*(\d+)\s*\n', comment='#'),
-                Quantity(
-                    'n_atoms', r'\s*ITEM:\s*NUMBER OF ATOMS\s*\n\s*(\d+)\s*\n', comment='#'),
-                Quantity(
-                    'pbc_cell', r'\s*ITEM: BOX BOUNDS\s*([\s\w]+)([\+\-\d\.eE\s]+)\n',
-                    str_operation=get_pbc_cell, comment='#'),
-                Quantity(
-                    'atoms_info', r's*ITEM:\s*ATOMS\s*([ \w]+\n)*?([\+\-eE\d\.\n ]+)\n*I*',
-                    str_operation=get_atoms_info, comment='#')
-            ]
-
-        parser = UnstructuredTextFileParser(self.mainfile, self._quantities)
-
-        self._results = {key: parser[key] for key in parser.keys() if parser[key]}
+        self._quantities = [
+            Quantity(
+                'time_step', r'\s*ITEM:\s*TIMESTEP\s*\n\s*(\d+)\s*\n', comment='#'),
+            Quantity(
+                'n_atoms', r'\s*ITEM:\s*NUMBER OF ATOMS\s*\n\s*(\d+)\s*\n', comment='#'),
+            Quantity(
+                'pbc_cell', r'\s*ITEM: BOX BOUNDS\s*([\s\w]+)([\+\-\d\.eE\s]+)\n',
+                str_operation=get_pbc_cell, comment='#'),
+            Quantity(
+                'atoms_info', r's*ITEM:\s*ATOMS\s*([ \w]+\n)*?([\+\-eE\d\.\n ]+)\n*I*',
+                str_operation=get_atoms_info, comment='#')
+        ]
 
     def with_trajectory(self):
-        if self._results is None:
-            return False
-
-        return self._results.get('atoms_info', None) is not None
+        return self.get('atoms_info') is not None
 
     @property
     def masses(self):
@@ -209,8 +216,11 @@ class TrajParser(Parser):
                 self._chemical_symbols[masses[i][0]] = self._reference_masses['symbols'][symbol_idx]
 
     def get_atom_labels(self, idx):
-        atoms_info = self.results.get('atoms_info')[idx]
-        atoms_type = atoms_info.get('type', None)
+        atoms_info = self.get('atoms_info')
+        if atoms_info is None:
+            return
+
+        atoms_type = atoms_info[idx].get('type')
         if atoms_type is None:
             return
 
@@ -222,9 +232,17 @@ class TrajParser(Parser):
         return atom_labels
 
     def get_positions(self, idx):
-        atoms_info = self.results.get('atoms_info')[idx]
+        atoms_info = self.get('atoms_info')
+        if atoms_info is None:
+            return
 
-        cell = self.results.get('pbc_cell')[idx][1]
+        atoms_info = atoms_info[idx]
+
+        cell = self.get('pbc_cell')
+        if cell is None:
+            return
+
+        cell = cell[idx][1]
 
         if 'xs' in atoms_info and 'ys' in atoms_info and 'zs' in atoms_info:
             positions = np.array([atoms_info['xs'], atoms_info['ys'], atoms_info['zs']]).T
@@ -240,30 +258,41 @@ class TrajParser(Parser):
 
                 positions += positions_img * np.linalg.norm(cell, axis=1)
 
-        return positions
+        return pint.Quantity(positions, self._units.get('distance', None))
 
     def get_velocities(self, idx):
-        atoms_info = self.results.get('atoms_info')[idx]
+        atoms_info = self.get('atoms_info')
+
+        if atoms_info is None:
+            return
+
+        atoms_info = atoms_info[idx]
 
         if 'vx' not in atoms_info or 'vy' not in atoms_info or 'vz' not in atoms_info:
             return
 
-        return np.array([atoms_info['vx'], atoms_info['vy'], atoms_info['vz']]).T
+        velocities = np.array([atoms_info['vx'], atoms_info['vy'], atoms_info['vz']]).T
+
+        return pint.Quantity(velocities, self._units.get('velocity', None))
 
     def get_forces(self, idx):
-        atoms_info = self.results.get('atoms_info')[idx]
+        atoms_info = self.get('atoms_info')
+
+        if atoms_info is None:
+            return
+
+        atoms_info = atoms_info[idx]
 
         if 'fx' not in atoms_info or 'fy' not in atoms_info or 'fz' not in atoms_info:
             return
 
-        return np.array([atoms_info['fx'], atoms_info['fy'], atoms_info['fz']]).T
+        forces = np.array([atoms_info['fx'], atoms_info['fy'], atoms_info['fz']]).T
+
+        return pint.Quantity(forces, self._units.get('force', None))
 
 
-class LogParser(Parser):
-    def __init__(self, mainfile, logger):
-        super().__init__(mainfile, logger)
-        self.logger = logger
-        self._thermo_data = None
+class LogParser(UnstructuredTextFileParser):
+    def __init__(self):
         self._commands = [
             'angle_coeff', 'angle_style', 'atom_modify', 'atom_style', 'balance',
             'bond_coeff', 'bond_style', 'bond_write', 'boundary', 'change_box', 'clear',
@@ -286,6 +315,36 @@ class LogParser(Parser):
             'write_data', 'write_dump', 'write_restart']
         self._interactions = [
             'atom', 'pair', 'bond', 'angle', 'dihedral', 'improper', 'kspace']
+        self._thermo_data = None
+        self._units = None
+        super().__init__(None)
+
+    def init_quantities(self):
+        def str_op(val):
+            val = val.split('#')[0]
+            val = val.replace('&\n', ' ').split()
+            val = val if len(val) > 1 else val[0]
+            return val
+
+        self._quantities = [
+            Quantity(
+                name, r'\n\s*%s\s+([\w\. \/\#\-]+)(\&\n[\w\. \/\#\-]*)*' % name,
+                str_operation=str_op, comment='#') for name in self._commands]
+
+        self._quantities.append(
+            Quantity('program_version', r'\s*LAMMPS\s*\(([\w ]+)\)\n', dtype=str)
+        )
+
+        self._quantities.append(
+            Quantity('finished', r'\s*Dangerous builds\s*=\s*(\d+)')
+        )
+
+    @property
+    def units(self):
+        if self._file_handler is None or self._units is None:
+            units_type = self.get('units', ['lj'])[0]
+            self._units = get_unit(units_type)
+        return self._units
 
     def get_thermodynamic_data(self):
         def str_operation(val):
@@ -319,10 +378,19 @@ class LogParser(Parser):
 
         self._thermo_data = list(thermo_data)[0] if thermo_data is not None else thermo_data
 
+        for key, val in self._thermo_data.items():
+            low_key = key.lower()
+            if low_key.startswith('e_') or low_key.endswith('eng'):
+                self._thermo_data[key] = val * self.units['energy']
+            elif low_key == 'press':
+                self._thermo_data[key] = val * self.units['pressure']
+            elif low_key == 'temp':
+                self._thermo_data[key] = val * self.units['temperature']
+
         return self._thermo_data
 
     def get_traj_files(self):
-        dump = self.results.get('dump', None)
+        dump = self.get('dump')
         if dump is None:
             self.logger.warn(
                 'Trajectory not specified in %s, will scan directory' % self.maindir)
@@ -338,7 +406,7 @@ class LogParser(Parser):
         return [os.path.join(self.maindir, f) for f in traj_files]
 
     def get_data_files(self):
-        read_data = self.results.get('read_data', None)
+        read_data = self.get('read_data')
         if read_data is None:
             self.logger.warn(
                 'Data file not specified in %s, will scan directory' % self.maindir)
@@ -351,11 +419,11 @@ class LogParser(Parser):
         return [os.path.join(self.maindir, f) for f in data_files]
 
     def get_pbc(self):
-        pbc = self.results.get('boundary', ['p', 'p', 'p'])
+        pbc = self.get('boundary', ['p', 'p', 'p'])
         return [v == 'p' for v in pbc]
 
     def get_program_version(self):
-        version = self.results.get('program_version', [''])[0]
+        version = self.get('program_version', [''])[0]
         return ' '.join(version)
 
     def get_sampling_method(self):
@@ -374,34 +442,38 @@ class LogParser(Parser):
         except IndexError:
             return {}
 
+        temp_unit = self.units['temperature']
+        press_unit = self.units['pressure']
+        time_unit = self.units['time']
+
         res = dict()
         if fix_style.lower() == 'nvt':
             try:
-                res['target_T'] = float(fix[5])
-                res['thermostat_tau'] = float(fix[6])
+                res['target_T'] = float(fix[5]) * temp_unit
+                res['thermostat_tau'] = float(fix[6]) * time_unit
             except Exception:
                 pass
 
         elif fix_style.lower() == 'npt':
             try:
-                res['target_T'] = float(fix[5])
-                res['thermostat_tau'] = float(fix[6])
-                res['target_P'] = float(fix[9])
-                res['barostat_tau'] = float(fix[10])
+                res['target_T'] = float(fix[5]) * temp_unit
+                res['thermostat_tau'] = float(fix[6]) * time_unit
+                res['target_P'] = float(fix[9]) * press_unit
+                res['barostat_tau'] = float(fix[10]) * time_unit
             except Exception:
                 pass
 
         elif fix_style.lower() == 'nph':
             try:
-                res['target_P'] = float(fix[5])
-                res['barostat_tau'] = float(fix[6])
+                res['target_P'] = float(fix[5]) * press_unit
+                res['barostat_tau'] = float(fix[6]) * time_unit
             except Exception:
                 pass
 
         elif fix_style.lower() == 'langevin':
             try:
-                res['target_T'] = float(fix[4])
-                res['langevin_gamma'] = float(fix[5])
+                res['target_T'] = float(fix[4]) * temp_unit
+                res['langevin_gamma'] = float(fix[5]) * time_unit
             except Exception:
                 pass
 
@@ -434,53 +506,22 @@ class LogParser(Parser):
         return styles_coeffs
 
     def finished_normally(self):
-        return self.get('finished', None) is not None
+        return self.get('finished') is not None
 
     def with_thermodynamics(self):
         return self._thermo_data is not None
 
-    def parse(self):
-        if self.mainfile is None:
-            return
 
-        if not os.path.isfile(self.mainfile):
-            return
+class LammpsParser(FairdiParser):
+    def __init__(self):
+        super().__init__(
+            name='parsers/lammps', code_name='LAMMPS', code_homepage='https://lammps.sandia.gov/',
+            domain='dft', mainfile_contents_re=r'^LAMMPS')
 
-        if self._quantities is None:
-            def str_op(val):
-                val = val.split('#')[0]
-                val = val.replace('&\n', ' ').split()
-                val = val if len(val) > 1 else val[0]
-                return val
-
-            self._quantities = [
-                Quantity(
-                    name, r'\n\s*%s\s+([\w\. \/\#\-]+)(\&\n[\w\. \/\#\-]*)*' % name,
-                    str_operation=str_op, comment='#') for name in self._commands]
-
-            self._quantities.append(
-                Quantity('program_version', r'\s*LAMMPS\s*\(([\w ]+)\)\n', dtype=str)
-            )
-
-            self._quantities.append(
-                Quantity('finished', r'\s*Dangerous builds\s*=\s*(\d+)')
-            )
-
-        parser = UnstructuredTextFileParser(self.mainfile, self._quantities)
-
-        self._results = {
-            key: parser[key] for key in parser.keys() if parser[key]}
-
-
-class LammpsOutput:
-    def __init__(self, filepath, archive, logger=None):
-        self.filepath = filepath
-        self.archive = archive
-        self.logger = logger if logger is not None else logging
-        self.log_parser = LogParser(filepath, self.logger)
-        self.traj_parser = TrajParser(None, self.logger)
-        self.data_parser = DataParser(None, self.logger)
-        self._converter = None
+        self._metainfo_env = m_env
+        self.log_parser = LogParser()
+        self.traj_parser = TrajParser()
+        self.data_parser = DataParser()
 
     def parse_thermodynamic_data(self):
         thermo_data = self.log_parser.get_thermodynamic_data()
@@ -521,16 +562,16 @@ class LammpsOutput:
                 if key in energy_keys_mapping:
                     sec_energy = sec_scc.m_create(section_energy_contribution)
                     sec_energy.energy_contibution_kind = energy_keys_mapping[key]
-                    sec_energy.energy_contribution_value = self._converter.Energy(val[n])
+                    sec_energy.energy_contribution_value = val[n]
 
                 elif key == 'toteng':
-                    sec_scc.energy_method_current = self._converter.Energy(val[n])
+                    sec_scc.energy_method_current = val[n]
 
                 elif key == 'press':
-                    sec_scc.pressure = self._converter.Press(val[n])
+                    sec_scc.pressure = val[n]
 
                 elif key == 'temp':
-                    sec_scc.temperature = self._converter.Temp(val[n])
+                    sec_scc.temperature = val[n]
 
                 elif key == 'step':
                     sec_scc.time_step = int(val[n])
@@ -549,7 +590,10 @@ class LammpsOutput:
 
         run_style = self.log_parser.get('run_style', ['verlet'])[0]
         run = self.log_parser.get('run', [0])[0]
-        timestep = self._converter.Time(self.log_parser.get('timestep', [0])[0])
+
+        units = self.log_parser.get('units', ['lj'])[0]
+        time_unit = get_unit(units, 'time')
+        timestep = self.log_parser.get('timestep', [0], unit=time_unit)[0]
         sampling_method, ensemble_type = self.log_parser.get_sampling_method()
 
         sec_sampling_method.x_lammps_integrator_type = run_style
@@ -561,30 +605,23 @@ class LammpsOutput:
         thermo_settings = self.log_parser.get_thermostat_settings()
         target_T = thermo_settings.get('target_T', None)
         if target_T is not None:
-            target_T = self._converter.Temp(target_T)
             sec_sampling_method.x_lammps_thermostat_target_temperature = target_T
         thermostat_tau = thermo_settings.get('thermostat_tau', None)
         if thermostat_tau is not None:
-            thermostat_tau = self._converter.Time(thermostat_tau)
             sec_sampling_method.x_lammps_thermostat_tau = thermostat_tau
         target_P = thermo_settings.get('target_P', None)
         if target_P is not None:
-            target_P = self._converter.Press(target_P)
             sec_sampling_method.x_lammps_barostat_target_pressure = target_P
         barostat_tau = thermo_settings.get('barostat_P', None)
         if barostat_tau is not None:
-            barostat_tau = self._converter.Time(barostat_tau)
             sec_sampling_method.x_lammps_barostat_tau = barostat_tau
         langevin_gamma = thermo_settings.get('langevin_gamma', None)
         if langevin_gamma is not None:
-            langevin_gamma = self._converter.Time(langevin_gamma)
             sec_sampling_method.x_lammps_langevin_gamma = langevin_gamma
 
-    def parse_system(self, trajfile):
+    def parse_system(self):
         sec_run = self.archive.section_run[-1]
 
-        self.traj_parser.mainfile = trajfile
-
         pbc_cell = self.traj_parser.get('pbc_cell', [])
         n_atoms = self.traj_parser.get('n_atoms', [])
 
@@ -600,11 +637,14 @@ class LammpsOutput:
             else:
                 create_scc = False
 
+        distance_unit = self.log_parser.units.get('distance', None)
+        self.traj_parser._units = self.log_parser.units
+
         for i in range(len(pbc_cell)):
             sec_system = sec_run.m_create(section_system)
             sec_system.number_of_atoms = n_atoms[i]
             sec_system.configuration_periodic_dimensions = pbc_cell[i][0]
-            sec_system.simulation_cell = self._converter.Distance(pbc_cell[i][1])
+            sec_system.simulation_cell = pbc_cell[i][1] * distance_unit
             sec_system.atom_positions = self.traj_parser.get_positions(i)
             atom_labels = self.traj_parser.get_atom_labels(i)
             if atom_labels is None:
@@ -613,7 +653,7 @@ class LammpsOutput:
 
             velocities = self.traj_parser.get_velocities(i)
             if velocities is not None:
-                sec_system.atom_velocities = self._converter.Velocity(velocities)
+                sec_system.atom_velocities = velocities
 
             forces = self.traj_parser.get_forces(i)
             if forces is not None:
@@ -622,13 +662,11 @@ class LammpsOutput:
                 else:
                     sec_scc = sec_sccs[i]
 
-                sec_scc.atom_forces = self._converter.Force(forces)
+                sec_scc.atom_forces = forces
 
-    def parse_topology(self, datafile):
+    def parse_topology(self):
         sec_run = self.archive.section_run[-1]
 
-        self.data_parser.mainfile = datafile
-
         masses = self.data_parser.get('Masses', None)
 
         self.traj_parser.masses = masses
@@ -647,10 +685,45 @@ class LammpsOutput:
             sec_interaction.interaction_kind = str(interaction[0])
             sec_interaction.interaction_parameters = [list(a) for a in interaction[1]]
 
-    def parse(self):
+    def parse_input(self):
+        sec_run = self.archive.section_run[-1]
+        sec_input_output_files = sec_run.m_create(x_lammps_section_input_output_files)
+
+        if self.data_parser.mainfile is not None:
+            sec_input_output_files.x_lammps_inout_file_data = os.path.basename(
+                self.data_parser.mainfile)
+
+        if self.traj_parser.mainfile is not None:
+            sec_input_output_files.x_lammps_inout_file_trajectory = os.path.basename(
+                self.traj_parser.mainfile)
+
+        sec_control_parameters = sec_run.m_create(x_lammps_section_control_parameters)
+        keys = self.log_parser._commands
+        for key in keys:
+            val = self.log_parser.get(key, None)
+            if val is None:
+                continue
+            val = val[0] if len(val) == 1 else val
+            key = 'x_lammps_inout_control_%s' % key.replace('_', '').replace('/', '').lower()
+            if hasattr(sec_control_parameters, key):
+                if isinstance(val, list):
+                    val = ' '.join([str(v) for v in val])
+                setattr(sec_control_parameters, key, str(val))
+
+    def _init_parsers(self):
+        self.log_parser.mainfile = self.filepath
+        self.log_parser.logger = self.logger
+        self.traj_parser.logger = self.logger
+        self.data_parser.logger = self.logger
+
+    def parse(self, filepath, archive, logger):
+        self.filepath = filepath
+        self.archive = archive
+        self.logger = logger if logger is not None else logging
+
+        self._init_parsers()
+
         sec_run = self.archive.m_create(section_run)
-        # TODO Warning! lj units are not supported in LammpsCommon!
-        self._converter = converter(self.log_parser.get('units', ['lj'])[0])
 
         # parse basic
         sec_run.program_name = 'LAMMPS'
@@ -663,17 +736,22 @@ class LammpsOutput:
         # specified in log file, otherwise will scan directory
         data_files = self.log_parser.get_data_files()
         for data_file in data_files:
-            self.parse_topology(data_file)
+            self.data_parser.mainfile = data_file
+            self.parse_topology()
 
         # parse all trajectorty files associated with calculation, normally only one
         # specified in log file, otherwise will scan directory
         traj_files = self.log_parser.get_traj_files()
         for traj_file in traj_files:
-            self.parse_system(traj_file)
+            self.traj_parser.mainfile = traj_file
+            self.parse_system()
 
         # parse thermodynamic data from log file
         self.parse_thermodynamic_data()
 
+        # include input controls from log file
+        self.parse_input()
+
         # create workflow
         if sec_run.section_sampling_method[0].sampling_method:
             sec_workflow = self.archive.m_create(Workflow)
