@@ -152,15 +152,14 @@ class section_scf_iteration(public.section_scf_iteration):
         a_legacy=LegacyDefinition(name='x_wien2k_iteration_number'))
 
     x_wien2k_nr_of_independent_atoms = Quantity(
-        type=np.dtype(np.int32),
-        shape=[],
+        type=int,
         description='''
         number of independent atoms in the cell
         ''',
         a_legacy=LegacyDefinition(name='x_wien2k_nr_of_independent_atoms'))
 
     x_wien2k_potential_option = Quantity(
-        type=np.dtype(np.int32),
+        type=str,
         shape=[],
         description='''
         exchange correlation potential option
@@ -216,23 +215,12 @@ class section_scf_iteration(public.section_scf_iteration):
         ''',
         a_legacy=LegacyDefinition(name='x_wien2k_unit_cell_volume_bohr3'))
 
-    x_wien2k_energy_total = Quantity(
-        type=np.dtype(np.float64),
-        shape=[],
-        unit='joule',
+    x_wien2k_spinpolarization = Quantity(
+        type=str,
         description='''
-        energy total
+        spinpolarization treatment
         ''',
-        a_legacy=LegacyDefinition(name='x_wien2k_energy_total'))
-
-    x_wien2k_fermi_ene = Quantity(
-        type=np.dtype(np.float64),
-        shape=[],
-        unit='joule',
-        description='''
-        Fermi energy
-        ''',
-        a_legacy=LegacyDefinition(name='x_wien2k_fermi_ene'))
+        a_legacy=LegacyDefinition(name='x_wien2k_spinpolarization'))
 
     x_wien2k_noe = Quantity(
         type=np.dtype(np.float64),
@@ -370,27 +358,27 @@ class section_scf_iteration(public.section_scf_iteration):
 
     x_wien2k_for_x_gl = Quantity(
         type=np.dtype(np.float64),
-        shape=[],
+        shape=['x_wien2k_nr_of_independent_atoms'],
         description='''
-        force on atom xx in mRy/bohr (in the global coordinate system of the unit cell (in
+        force on inequivalent atom xx (in the global coordinate system of the unit cell (in
         the same way as the atomic positions are specified)): Fx
         ''',
         a_legacy=LegacyDefinition(name='x_wien2k_for_x_gl'))
 
     x_wien2k_for_y_gl = Quantity(
         type=np.dtype(np.float64),
-        shape=[],
+        shape=['x_wien2k_nr_of_independent_atoms'],
         description='''
-        force on atom xx in mRy/bohr (in the global coordinate system of the unit cell (in
+        force on inequivalent atom xx in (in the global coordinate system of the unit cell (in
         the same way as the atomic positions are specified)): Fy
         ''',
         a_legacy=LegacyDefinition(name='x_wien2k_for_y_gl'))
 
     x_wien2k_for_z_gl = Quantity(
         type=np.dtype(np.float64),
-        shape=[],
+        shape=['x_wien2k_nr_of_independent_atoms'],
         description='''
-        force on atom xx in mRy/bohr (in the global coordinate system of the unit cell (in
+        force on inequivalent atom xx in (in the global coordinate system of the unit cell (in
         the same way as the atomic positions are specified)): Fz
         ''',
         a_legacy=LegacyDefinition(name='x_wien2k_for_z_gl'))
@@ -403,6 +391,14 @@ class section_scf_iteration(public.section_scf_iteration):
         ''',
         a_legacy=LegacyDefinition(name='x_wien2k_atom_nr'))
 
+    x_wien2k_atom_mult = Quantity(
+        type=np.dtype(np.int32),
+        shape=['x_wien2k_nr_of_independent_atoms'],
+        description='''
+        atom multiplicity
+        ''',
+        a_legacy=LegacyDefinition(name='x_wien2k_atom_mult'))
+
     x_wien2k_sphere_nr = Quantity(
         type=str,
         shape=[],
@@ -738,16 +734,6 @@ class section_method(public.section_method):
         ''',
         a_legacy=LegacyDefinition(name='x_wien2k_smearing_kind'))
 
-    x_wien2k_smearing_width = Quantity(
-        type=np.dtype(np.float64),
-        shape=[],
-        unit='joule',
-        description='''
-        specifies the width of the broadening, if efmod is set to ALL, eval specifies the
-        upper limit of the energy window; in in2
-        ''',
-        a_legacy=LegacyDefinition(name='x_wien2k_smearing_width'))
-
     x_wien2k_in2_gmax = Quantity(
         type=np.dtype(np.float64),
         shape=[],
@@ -761,29 +747,5 @@ class section_method(public.section_method):
         repeats=True,
         a_legacy=LegacyDefinition(name='x_wien2k_section_XC'))
 
-    x_wien2k_in2c_switch = Quantity(
-        type=str,
-        a_legacy=LegacyDefinition(name='x_wien2k_in2c_switch'))
-
-    x_wien2k_in2c_emin = Quantity(
-        type=np.dtype(np.float64),
-        a_legacy=LegacyDefinition(name='x_wien2k_in2c_emin'))
-
-    x_wien2k_in2c_ne = Quantity(
-        type=np.dtype(np.float64),
-        a_legacy=LegacyDefinition(name='x_wien2k_in2c_ne'))
-
-    x_wien2k_in2c_espermin = Quantity(
-        type=np.dtype(np.float64),
-        a_legacy=LegacyDefinition(name='x_wien2k_in2c_espermin'))
-
-    x_wien2k_in2c_esper0 = Quantity(
-        type=np.dtype(np.float64),
-        a_legacy=LegacyDefinition(name='x_wien2k_in2c_esper0'))
-
-    x_wien2k_in2c_gmax = Quantity(
-        type=np.dtype(np.float64),
-        a_legacy=LegacyDefinition(name='x_wien2k_in2c_gmax'))
-
 
 m_package.__init_metainfo__()
