@@ -8,6 +8,7 @@ import sys
 from nomadcore.simple_parser import mainFunction, AncillaryParser, CachingLevel
 from nomadcore.simple_parser import SimpleMatcher as SM
 from nomadcore.local_meta_info import loadJsonFile, InfoKindEl
+from nomadcore.unit_conversion import unit_conversion
 import wien2kparser.wien2k_parser_struct as wien2k_parser_struct
 import wien2kparser.wien2k_parser_in0 as wien2k_parser_in0
 import wien2kparser.wien2k_parser_in1 as wien2k_parser_in1
@@ -21,6 +22,7 @@ from nomad.parsing.legacy import CoESimpleMatcherParser
 ################################################################
 
 # Copyright 2016-2018 Daria M. Tomecka, Fawzi Mohamed
+# Copyright 2020 Pavel Ondračka
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -53,6 +55,9 @@ class Wien2kContext(object):
         self.secMethodIndex = None
         self.secSystemIndex = None
         self.scfIterNr = 0
+        self.spinPol = None
+        self.eTot = None
+        self.forces = None
 
     def startedParsing(self, path, parser):
         """called when parsing starts"""
@@ -126,42 +131,142 @@ class Wien2kContext(object):
 
 
     def onClose_section_single_configuration_calculation(self, backend, gIndex, section):
+
        # write number of SCF iterations
         backend.addValue('number_of_scf_iterations', self.scfIterNr)
         # write the references to section_method and section_system
         backend.addValue('single_configuration_to_calculation_method_ref', self.secMethodIndex)
         backend.addValue('single_configuration_calculation_to_system_ref', self.secSystemIndex)
+        fromR = unit_conversion.convert_unit_function("rydberg", "J")
+
+        if self.eTot is not None:
+            backend.addValue("energy_total", self.eTot)
+
+        if self.forces is not None:
+           backend.addArrayValues('atom_forces', self.forces)
+
+        mainFile = self.parser.fIn.fIn.name
+
+        eigvalKpoint=[]
+        eigvalKpointMult=[]
+        eigvalVal=[[]]
+
+        # Read eigenvalues from the energy files
+        # this can be either case.energy for non spinpolarized serial calculation
+        # case.energyup and case.energydn for spinpolarized serial calculation
+        # case.energy_x files for non spinpolarized parallel calculation
+        # case.energyup_x and case.energydn_x for spinpolarized parallel calculation
+        if self.spinPol == None:
+            suffixes = ["", "up", "dn"]
+        elif self.spinPol:
+            suffixes = ["up", "dn"]
+        else:
+            suffixes = [""]
+        spin = 0
+
+        for suf in suffixes:
+            for i in range(1000):
+                paraIdx = ""
+                if i != 0:
+                    paraIdx = "_" + str(i)
+                fName = mainFile[:-4] + ".energy" + suf + paraIdx
+                if os.path.exists(fName) and os.stat(fName).st_size > 0:
+                    with open(fName) as g:
+                        if suf == "dn":
+                            spin = 1
+                            if len(eigvalVal) == 1:
+                                eigvalVal.append([])
+                        else:
+                            spin = 0
+                        eigvalToRead=0
+                        for l in g:
+                            if len(l) > 90:
+                                continue
+                            #there are no spaces sometime between the numbers but the format is fixed
+                            if eigvalToRead == 0:
+                                kx = float(l[0:19])
+                                ky = float(l[19:38])
+                                kz = float(l[38:57])
+                                eigvalKpointInd = int(l[57:67])
+                                if len(eigvalKpoint) == eigvalKpointInd - 1:
+                                    eigvalKpoint.append([kx, ky, kz])
+                                    eigvalKpointMult.append([int(float(l[79:84]))])
+                                if len(eigvalVal[spin]) != eigvalKpointInd -1:
+                                    break
+                                    #raise Exception("Found old eigenvalues for the current k-point while reading from %s,\
+                                    #        possible mixing of serial nad parallel runs?" % fName)
+                                eigvalToRead = int(l[73:79])
+                                eigvalVal[spin].append([])
+                            else:
+                                eigvalVal[spin][-1].append(fromR(float(l[12:31])))
+                                eigvalToRead -= 1
+                elif i > 0:
+                    break
+        if eigvalVal[0]: 
+            eigvalGIndex = backend.openSection("section_eigenvalues")
+            backend.addArrayValues("eigenvalues_values", np.asarray(eigvalVal))
+            backend.addArrayValues("eigenvalues_kpoints", np.asarray(eigvalKpoint))
+            backend.addArrayValues("eigenvalues_kpoints_multiplicity", np.asarray(eigvalKpointMult))
+            backend.closeSection("section_eigenvalues",eigvalGIndex)
+
+        #iterate over all dos files, for now just the total dos is supported
+        spin = 0
+        DOS = [[]]
+        ene = []
+        eneNorm = []
+        for suf in suffixes:
+            for n in range(1,50):
+                fName = mainFile[:-4] + ".dos" + str(n) + suf
+                if os.path.exists(fName):
+                    #read the DOS file
+                    with open(fName) as f:
+                        if suf == "dn":
+                            spin = 1
+                            if len(DOS) == 1:
+                                DOS.append([])
+                        ene = []
+                        eneNorm = []
+                        Ef = 0.0
+                        totDOScol = 0
+                        for i,l in enumerate(f):
+                            l = l.split()
+                            if i == 1:
+                                Ef = float(l[1])
+                            if i == 2:
+                                #find out which column is the total DOS
+                                for j,w in enumerate(l):
+                                    if "total-DOS" in w or "TOTAL" in w:
+                                        totDOScol = j - 1
+                                        break
+                                # there is no total DOS column here
+                                if totDOScol == 0:
+                                    break
+                            if i > 2:
+                                ene.append(fromR(float(l[0])))
+                                eneNorm.append(fromR(float(l[0]) - Ef))
+                                DOS[spin].append(float(l[totDOScol]) / fromR(1.0))
+                    # we found the total DOS, ignore the rest of files
+                    if DOS[spin]:
+                        break
+                else:
+                    break
+        if DOS[0]:
+            DOSGIndex = backend.openSection("section_dos")
+            backend.addArrayValues("dos_energies", np.asarray(ene))
+            backend.addArrayValues("dos_energies_normalized", np.asarray(eneNorm))
+            backend.addArrayValues("dos_values", np.asarray(DOS))
+            backend.addValue("dos_kind", "electronic")
+            backend.addValue("number_of_dos_values", len(DOS[0]))
+            backend.closeSection("section_dos",DOSGIndex)
 
 
     def onClose_section_system(self, backend, gIndex, section):
 
-        #backend.addValue("smearing_kind", x_fleur_smearing_kind)
-        smearing_kind = section['x_wien2k_smearing_kind']
-        if smearing_kind is not None:
-        #    value = ''
-            backend.addValue('x_wien2k_smearing_kind', value)
-
-
-        smearing_width = section['x_wien2k_smearing_width']
-        if smearing_width is not None:
-        #    value = ''
-            backend.addValue('x_wien2k_smearing_width', value)
-
         #   atom labels
         atom_labels = section['x_wien2k_atom_name']
         if atom_labels is not None:
            backend.addArrayValues('atom_labels', np.asarray(atom_labels))
 
-        # atom force
-        atom_force = []
-        for i in ['x', 'y', 'z']:
-            api = section['x_wien2k_for_' + i]
-            if api is not None:
-               atom_force.append(api)
-        if atom_force:
-            # need to transpose array since its shape is [number_of_atoms,3] in\the metadata
-           backend.addArrayValues('atom_forces', np.transpose(np.asarray(atom_force)))
-
         # Parse the structure file
         mainFile = self.parser.fIn.fIn.name
         fName = mainFile[:-4] + ".struct"
@@ -191,12 +296,68 @@ class Wien2kContext(object):
                     superContext = structSuperContext)
                 structParser.parseFile(fin)
 
+    def onClose_section_method(self, backend, gIndex, section):
+        #Trigger called when section_method is closed.
+
+        if self.spinPol is not None:
+            if self.spinPol:
+                backend.addValue("number_of_spin_channels", 2)
+            else:
+                backend.addValue("number_of_spin_channels", 1)
+
+        mainFile = self.parser.fIn.fIn.name
+
+        #read kpoints from the klist file
+        klistFile = mainFile[:-4] + ".klist"
+        if os.path.exists(klistFile):
+            with open(klistFile) as f:
+                kMeshWeights=[]
+                weightsSum=0.0
+                kMeshPoints=[]
+                for l in f:
+                    tmp = l.split()
+                    if len(tmp) >= 6:
+                        kMeshWeights.append(float(tmp[5]))
+                        weightsSum += kMeshWeights[-1]
+                        kMeshPoints.append([float(tmp[1])/float(tmp[4]), float(tmp[2])/float(tmp[4]), float(tmp[3])/float(tmp[4])])
+                    elif tmp[0] == 'END': break
+                kMeshWeights = [w/weightsSum for w in kMeshWeights]
+                backend.addArrayValues("k_mesh_points", np.asarray(kMeshPoints))
+                backend.addArrayValues("k_mesh_weights", np.asarray(kMeshWeights))
+
     def onClose_section_scf_iteration(self, backend, gIndex, section):
         #Trigger called when section_scf_iteration is closed.
 
         # count number of SCF iterations
         self.scfIterNr += 1
 
+        sp = section["x_wien2k_spinpolarization"]
+        if sp is not None:
+            if "NON" in sp[0]:
+                self.spinPol = False
+            else:
+                self.spinPol = True
+        
+        eTot = section["energy_total_scf_iteration"]
+        if eTot is not None:
+            self.eTot = eTot[0]
+
+        # save the forces
+        fx = section['x_wien2k_for_x_gl']
+        fy = section['x_wien2k_for_y_gl']
+        fz = section['x_wien2k_for_z_gl']
+        if fx is not None and fy is not None and fz is not None:
+            backend.addArrayValues('x_wien2k_for_x_gl', fx)
+            backend.addArrayValues('x_wien2k_for_y_gl', fy)
+            backend.addArrayValues('x_wien2k_for_z_gl', fz)
+            # account for atom multiplicities
+            self.forces=[]
+            for i,a in enumerate(section['x_wien2k_atom_mult']):
+                for _ in range(a):
+                    self.forces.append([fx[i], fy[i], fz[i]])
+            print (self.forces)
+
+
 # description of the input
 mainFileDescription = SM(
     name = 'root',
@@ -224,24 +385,26 @@ mainFileDescription = SM(
                   subMatchers=[
                       SM(r":NATO\s*:\s*(?P<x_wien2k_nr_of_independent_atoms>[0-9]+)\s*INDEPENDENT AND\s*(?P<x_wien2k_total_atoms>[0-9]+)\s*TOTAL ATOMS IN UNITCELL"),
                       SM(r"\s*SUBSTANCE: (?P<x_wien2k_system_name>.*)"),
-                      SM(r":POT\s*:\s*POTENTIAL OPTION\s*(?P<x_wien2k_potential_option>[0-9]+)"),
+                      # older Wien2k versions used numerical identification for the potential
+                      # newer Wien2k versions allow to specificy separatelly the functional for calculation of both exchange and correlation potential and energy separatelly
+                      # the output can look like this, ":POT  : POTENTIAL OPTION EX_LDA EC_LDA VX_MBJ VC_LDA"
+                      SM(r":POT\s*:\s*POTENTIAL OPTION\s*(?P<x_wien2k_potential_option>[0-9 \w]+)"),
                       SM(r":LAT\s*:\s*LATTICE CONSTANTS=\s*(?P<x_wien2k_lattice_const_a>[0-9.]+)\s*(?P<x_wien2k_lattice_const_b>[0-9.]+)\s*(?P<x_wien2k_lattice_const_c>[0-9.]+)"),
                       SM(r":VOL\s*:\s*UNIT CELL VOLUME\s*=\s*(?P<x_wien2k_unit_cell_volume_bohr3>[0-9.]+)"),
-                      SM(r":RKM  : MATRIX SIZE (?P<x_wien2k_matrix_size>[0-9]+)\s*LOs:\s*(?P<x_wien2k_LOs>[0-9.]+)\s*RKM=\s*(?P<x_wien2k_rkm>[0-9.]+)\s*WEIGHT=\s*[0-9.]*\s*\w*:"),
+                      SM(r"\s*(?P<x_wien2k_spinpolarization>(NON-)?SPINPOLARIZED) CALCULATION\s*"),
+                      SM(r":RKM  : MATRIX SIZE\s*(?P<x_wien2k_matrix_size>[0-9]+)\s*LOs:\s*(?P<x_wien2k_LOs>[0-9.]+)\s*RKM=\s*(?P<x_wien2k_rkm>[0-9.]+)\s*WEIGHT=\s*[0-9.]*\s*\w*:"),
                       SM(r":KPT\s*:\s*NUMBER\s*OF\s*K-POINTS:\s*(?P<x_wien2k_nr_kpts>[-+0-9.]+)"),
-                      #SM(r":GMA\s*:\s*POTENTIAL\sAND\sCHARGE\sCUT-OFF\s*(?P<x_wien2k_cutoff>[0-9.]+)\s*Ry\*\*[0-9.]+"),
-                      SM(r":GMA\s*:\s*POTENTIAL\sAND\sCHARGE\sCUT-OFF\s*(?P<x_wien2k_cutoff>[0-9.]+)\s*Ry\W\W[0-9.]+"),
                       SM(r":GAP\s*:\s*(?P<x_wien2k_ene_gap__rydberg>[-+0-9.]+)\s*Ry\s*=\s*(?P<x_wien2k_ene_gap_eV>[-+0-9.]+)\s*eV\s*.*"),
                       SM(r":NOE\s*:\s*NUMBER\sOF\sELECTRONS\s*=\s*(?P<x_wien2k_noe>[0-9.]+)"),
-                      SM(r":FER\s*:\sF E R M I -\s\w*\W\w*\WM\W*=\s*(?P<energy_reference_fermi_iteration__rydberg>[-+0-9.]+)"),
-                      SM(r":GMA\s*:\s*POTENTIAL\sAND\sCHARGE\sCUT-OFF\s*[0-9.]+\s*Ry\W\W[0-9.]+"),
-                      SM(r":CHA(?P<x_wien2k_atom_nr>[-+0-9]+):\s*TOTAL\s*\w*\s*CHARGE INSIDE SPHERE\s*(?P<x_wien2k_sphere_nr>[-+0-9]+)\s*=\s*(?P<x_wien2k_tot_val_charge_sphere>[0-9.]+)",repeats = True),
-                      SM(r":CHA\s*:\s*TOTAL\s*\w*\s*CHARGE INSIDE\s*\w*\s*CELL\s=\s*(?P<x_wien2k_tot_val_charge_cell>[-+0-9.]+)"),
+                      SM(r":FER\s*:\sF E R M I - ENERGY\W\w*\W\w*M\W*=\s*(?P<energy_reference_fermi_iteration__rydberg>[-+0-9.]+)"),
+                      SM(r":GMA\s*:\s*POTENTIAL\sAND\sCHARGE\sCUT-OFF\s*(?P<x_wien2k_cutoff>[0-9.]+)\s*Ry\W\W[0-9.]+"),
+                      SM(r":POS[0-9]*:\s*ATOM\s*[-0-9.]+\s*X,Y,Z\s*=[-0-9. ]+MULT=\s*(?P<x_wien2k_atom_mult>[0-9]+)\s*ZZ=\s*[0-9.]+.*", repeats = True),
+                      #FIXME: this is repeating interleaved with the previous matcher and likely need its own submatcher 
+#                     SM(r":CHA(?P<x_wien2k_atom_nr>[-+0-9]+):\s*TOTAL\s*\w*\s*CHARGE INSIDE SPHERE\s*(?P<x_wien2k_sphere_nr>[-+0-9]+)\s*=\s*(?P<x_wien2k_tot_val_charge_sphere>[0-9.]+)",repeats = True),
+#                      SM(r":CHA\s*:\s*TOTAL\s*\w*\s*CHARGE INSIDE\s*\w*\s*CELL\s=\s*(?P<x_wien2k_tot_val_charge_cell>[-+0-9.]+)"),
                       SM(r":SUM\s*:\s*SUM OF EIGENVALUES\s*=\s*(?P<energy_sum_eigenvalues_scf_iteration__rydberg>[-+0-9.]+)"),
-                      SM(r":MMTOT: TOTAL MAGNETIC MOMENT IN CELL =\s*(?P<x_wien2k_mmtot>[-+0-9.]+)"),
-                      SM(r":MMINT: MAGNETIC MOMENT IN INTERSTITIAL =\s*(?P<x_wien2k_mmint>[-+0-9.]+)"),
-                      SM(r":MMI001: MAGNETIC MOMENT IN SPHERE 1 =\s*(?P<x_wien2k_mmi001>[-+0-9.]+)"),
                       SM(r":RTO(?P<x_wien2k_atom_nr>[-+0-9]+)\s*:\s*[0-9]+\s*(?P<x_wien2k_density_at_nucleus_valence>[-+0-9.]+)\s*(?P<x_wien2k_density_at_nucleus_semicore>[-+0-9.]+)\s*(?P<x_wien2k_density_at_nucleus_core>[-+0-9.]+)\s*(?P<x_wien2k_density_at_nucleus_tot>[0-9.]+)",repeats = True),
+                      #FIXME: followig matchers work just for cases without spin polarization
                       SM(r":NTO\s*:\s*\sTOTAL\s*INTERSTITIAL\s*CHARGE=\s*(?P<x_wien2k_tot_int_charge_nm>[-+0-9.]+)"),
                       SM(r":NTO(?P<x_wien2k_atom_nr>[-+0-9]+)[0-9]*:\s*\sTOTAL\s*CHARGE\s*IN\s*SPHERE\s*(?P<x_wien2k_sphere_nr>[-+0-9]+)\s*=\s*(?P<x_wien2k_tot_charge_in_sphere_nm>[-+0-9.]+)",repeats = True),
                       SM(r":DTO(?P<x_wien2k_atom_nr>[-+0-9]+)[0-9]*:\sTOTAL\s*DIFFERENCE\s*CHARGE\W*\w*\s*IN\s*SPHERE\s*(?P<x_wien2k_sphere_nr>[-+0-9]+)\s*=\s*(?P<x_wien2k_tot_diff_charge>[-+0-9.]+)", repeats = True),
@@ -249,9 +412,14 @@ mainFileDescription = SM(
                       SM(r":CTO\s*:\s*\sTOTAL\s*INTERSTITIAL\s*CHARGE=\s*(?P<x_wien2k_tot_int_charge>[-+0-9.]+)"),
                       SM(r":CTO(?P<x_wien2k_atom_nr>[-+0-9]+)[0-9]*:\s*\sTOTAL\s*CHARGE\s*IN\s*SPHERE\s*(?P<x_wien2k_sphere_nr>[-+0-9]+)\s*=\s*(?P<x_wien2k_tot_charge_in_sphere>[-+0-9.]+)",repeats = True),
 #                      SM(r":NEC(?P<x_wien2k_necnr>[-+0-9]+)\s*:\s*NUCLEAR AND ELECTRONIC CHARGE\s*(?P<x_wien2k_nuclear_charge>[-+0-9.]+)\s*(?P<x_wien2k_electronic_charge>[0-9.]+)",repeats = True),
+                      SM(r":MMINT:\s*MAGNETIC MOMENT IN INTERSTITIAL\s*=\s*(?P<x_wien2k_mmint>[-+0-9.]+)"),
+                      #FIXME: read for all spheres
+                      SM(r":MMI001:\s*MAGNETIC MOMENT IN SPHERE\s*1\s*=\s*(?P<x_wien2k_mmi001>[-+0-9.]+)"),
+                      # its not clear if the old and new MMTOT (total magnetic moment in cell vs spin magnetic moment in cell) are the same thing?
+                      SM(r":MMTOT:\s*(TOTAL|SPIN) MAGNETIC MOMENT IN CELL\s*=\s*(?P<x_wien2k_mmtot>[-+0-9.]+)"),
                       SM(r":ENE\s*:\s*\W*\w*\W*\s*TOTAL\s*ENERGY\s*IN\s*Ry\s*=\s*(?P<energy_total_scf_iteration__rydberg>[-+0-9.]+)"),
                       SM(r":FOR[0-9]*:\s*(?P<x_wien2k_atom_nr>[0-9]+).ATOM\s*(?P<x_wien2k_for_abs>[0-9.]+)\s*(?P<x_wien2k_for_x>[-++0-9.]+)\s*(?P<x_wien2k_for_y>[-+0-9.]+)\s*(?P<x_wien2k_for_z>[-+0-9.]+)\s*partial\sforces", repeats = True),
-                      SM(r":FGL[0-9]*:\s*(?P<x_wien2k_atom_nr>[0-9]+).ATOM\s*(?P<x_wien2k_for_x_gl>[-+0-9.]+)\s*(?P<x_wien2k_for_y_gl>[-+0-9.]+)\s*(?P<x_wien2k_for_z_gl>[-+0-9.]+)\s*partial\sforces", repeats = True)
+                      SM(r":FGL[0-9]*:\s*(?P<x_wien2k_atom_nr>[0-9]+).ATOM\s*(?P<x_wien2k_for_x_gl__mrydberg_bohr_1>[-+0-9.]+)\s*(?P<x_wien2k_for_y_gl__mrydberg_bohr_1>[-+0-9.]+)\s*(?P<x_wien2k_for_z_gl__mrydberg_bohr_1>[-+0-9.]+)\s*total\sforces", repeats = True),
                   ]
               )
            ]
