@@ -45,6 +45,15 @@ class Wien2kIn2Context(object):
         # allows to reset values if the same superContext is used to parse different files
         self.initialize_values()
 
+    def onClose_section_method(self, backend, gIndex, section):
+        smearing_kind = section['x_wien2k_smearing_kind'][0]
+        if smearing_kind is not None:
+            if 'TETRA' in smearing_kind:
+                backend.addValue('smearing_kind', 'tetrahedra')
+            elif 'TEMP' in smearing_kind:
+                backend.addValue('smearing_kind', 'fermi')
+            elif 'GAUSS' in smearing_kind:
+                backend.addValue('smearing_kind', 'gaussian')
 
 # description of the input
 def buildIn2Matchers():
@@ -56,10 +65,10 @@ def buildIn2Matchers():
     subMatchers = [
 #        SM(name = 'systemName',
 #          startReStr = r"(?P<x_wien2k_system_nameIn>.*)"),
-        SM(r"\s*(?P<x_wien2k_in2c_switch>[A-Z]+)\s*.*"),
-        SM(r"\s*(?P<x_wien2k_in2c_emin>[-+0-9.]+)\s*(?P<x_wien2k_in2c_ne>[-+0-9.]+)\s*(?P<x_wien2k_in2c_espermin>[-+0-9.]+)\s*(?P<x_wien2k_in2c_esper0>[-+0-9.]+)\s*.*"),
+        SM(r"\s*(?P<x_wien2k_in2_switch>[A-Z]+)\s*.*"),
+        SM(r"\s*(?P<x_wien2k_in2_emin>[-+0-9.]+)\s*(?P<x_wien2k_in2_ne>[-+0-9.]+)\s*(?P<x_wien2k_in2_espermin>[-+0-9.]+)\s*(?P<x_wien2k_in2_esper0>[-+0-9.]+)\s*.*"),
         SM(r"\s*(?P<x_wien2k_smearing_kind>[A-Z]+)\s*\s*(?P<smearing_width__rydberg>[-+0-9.]+)\s*.*"),
-        SM(r"\s*(?P<x_wien2k_in2c_gmax>[-+0-9.]+)\s*GMAX")
+        SM(r"\s*(?P<x_wien2k_in2_gmax>[-+0-9.]+)\s*GMAX")
 
     ])
 
