@@ -1,16 +1,20 @@
-# Copyright 2018 Markus Scheidgen
+#
+# Copyright The NOMAD Authors.
+#
+# This file is part of NOMAD. See https://nomad-lab.eu for further info.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#   http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an"AS IS" BASIS,
+# distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 
 '''
 The archive API of the nomad@FAIRDI APIs. This API is about serving processed
@@ -21,7 +25,6 @@ from typing import Dict, Any
 from io import BytesIO
 from flask import request, g
 from flask_restplus import abort, Resource, fields
-import json
 import orjson
 import urllib.parse
 
@@ -159,6 +162,15 @@ class ArchiveDownloadResource(Resource):
                 for entry in calcs:
                     upload_id = entry['upload_id']
                     calc_id = entry['calc_id']
+
+                    manifest = {
+                        calc_id: {
+                            key: entry[key]
+                            for key in ArchiveDownloadResource.manifest_quantities
+                            if entry.get(key) is not None
+                        }
+                    }
+
                     if upload_files is None or upload_files.upload_id != upload_id:
                         if upload_files is not None:
                             upload_files.close()
@@ -178,40 +190,21 @@ class ArchiveDownloadResource(Resource):
                             option=orjson.OPT_INDENT_2 | orjson.OPT_NON_STR_KEYS))
 
                         yield (
-                            '%s.%s' % (calc_id, 'json'), calc_id,
+                            '%s.%s' % (calc_id, 'json'), calc_id, manifest,
                             lambda calc_id: f,
                             lambda calc_id: f.getbuffer().nbytes)
 
-                    manifest[calc_id] = {
-                        key: entry[key]
-                        for key in ArchiveDownloadResource.manifest_quantities
-                        if entry.get(key) is not None
-                    }
-
                 if upload_files is not None:
                     upload_files.close()
 
-                try:
-                    manifest_contents = json.dumps(manifest).encode('utf-8')
-                except Exception as e:
-                    manifest_contents = json.dumps(
-                        dict(error='Could not create the manifest: %s' % (e))).encode('utf-8')
-                    common.logger.error(
-                        'could not create raw query manifest', exc_info=e)
-
-                yield (
-                    'manifest.json', 'manifest',
-                    lambda *args: BytesIO(manifest_contents),
-                    lambda *args: len(manifest_contents))
-
             except Exception as e:
-                common.logger.warning(
+                common.logger.error(
                     'unexpected error while streaming raw data from query',
                     exc_info=e,
                     query=urllib.parse.urlencode(request.args, doseq=True))
 
         return streamed_zipfile(
-            generator(), zipfile_name='nomad_archive.zip', compress=compress)
+            generator(), zipfile_name='nomad_archive.zip', compress=compress, manifest=dict())
 
 
 _archive_query_model = api.inherit('ArchiveSearch', search_model, {
