@@ -1,16 +1,20 @@
-# Copyright 2018 Markus Scheidgen
+#
+# Copyright The NOMAD Authors.
+#
+# This file is part of NOMAD. See https://nomad-lab.eu for further info.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#   http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an"AS IS" BASIS,
+# distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 
 """
 API for retrieving material information.
@@ -24,6 +28,7 @@ from flask_restplus import Resource, abort, fields, marshal
 from flask import request, g
 from elasticsearch_dsl import Search, Q, A
 from elasticsearch_dsl.utils import AttrDict
+import ase.data
 
 from nomad import config, infrastructure, search
 from nomad.files import UploadFiles
@@ -236,7 +241,8 @@ def get_es_doc_values(es_doc, mapping, keys=None):
     for key in keys:
         es_key = mapping[key]
         value = rgetattr(es_doc, es_key)
-        result[key] = value
+        if value is not None:
+            result[key] = value
 
     return result
 
@@ -321,11 +327,10 @@ material_result = api.model("material_result", {
 @ns.route("/materials/<string:material_id>")
 class EncMaterialResource(Resource):
     @api.response(404, "The material does not exist")
-    @api.response(200, "Metadata send", fields.Raw)
-    @api.doc("get_material")
+    @api.response(200, "Metadata send", material_result)
     @api.expect(material_query)
     @api.marshal_with(material_result, skip_none=True)
-    @api.param("material_id", "28 character identifier for the material.")
+    @api.doc("get_material", params={"material_id": "28 character identifier for the material."})
     @authenticate()
     def get(self, material_id):
         """Used to retrieve basic information related to a material.
@@ -384,29 +389,29 @@ class EncMaterialResource(Resource):
                 except AttributeError:
                     pass
             if similarity:
-                result["similarity"] = similarity
+                result["similarity"] = sorted(similarity, key=lambda x: x["value"], reverse=True)
 
         return result, 200
 
 
 re_formula = re.compile(r"([A-Z][a-z]?)(\d*)")
 range_query = api.model("range_query", {
-    "max": fields.Float,
-    "min": fields.Float,
+    "max": fields.Float(min=0),
+    "min": fields.Float(min=0),
 })
 materials_query = api.model("materials_input", {
     "search_by": fields.Nested(api.model("search_query", {
-        "exclusive": fields.Boolean(default=False),
-        "formula": fields.String,
-        "element": fields.String,
-        "page": fields.Integer(default=1, description="Requested page number, indexing starts from 1."),
-        "per_page": fields.Integer(default=25, description="Number of results per page."),
+        "exclusive": fields.Boolean(default=False, description="Set to True to enable exclusive element search."),
+        "formula": fields.String(description="Chemical formula of the material as a string. Use a single continuous string with capitalized element names and integer numbers for the element multiplicity. The order of elements does not matter.", example="TiO2"),
+        "elements": fields.List(fields.String(enum=ase.data.chemical_symbols[1:]), description="List of chemical species that the material should include. Use capitalized element name abbreviations.", example=["Ti", "O"]),
+        "page": fields.Integer(default=1, min=1, description="Requested page number, indexing starts from 1.", example=1),
+        "per_page": fields.Integer(default=25, min=1, description="Number of results per page.", example=10),
         "restricted": fields.Boolean(default=False, description="Select to restrict the query to individual calculations. If not selected, the query will combine results from several different calculations."),
     })),
     "material_type": fields.List(fields.String(enum=list(Material.material_type.type)), description=Material.material_type.description),
     "material_name": fields.List(fields.String, description=Material.material_name.description),
     "structure_type": fields.List(fields.String, description=Bulk.structure_type.description),
-    "space_group_number": fields.List(fields.Integer, description=Bulk.space_group_number.description),
+    "space_group_number": fields.List(fields.Integer, min=1, max=230, description=Bulk.space_group_number.description),
     "crystal_system": fields.List(fields.String(enum=list(Bulk.crystal_system.type)), description=Bulk.crystal_system.description),
     "band_gap": fields.Nested(range_query, description="Band gap range in eV.", allow_null=True),
     "has_band_structure": fields.Boolean(description="Set to True if electronic band structure needs to be available for this material."),
@@ -481,33 +486,24 @@ class EncMaterialsResource(Resource):
             for f in calc_filters:
                 s.add_calculation_filter(f)
 
-        # if data["functional_type"] is not None: s.add_calculation_filter(Q("terms", calculations__method__functional_type=data["functional_type"]))
-        # if data["basis_set"] is not None: s.add_calculation_filter(Q("terms", calculations__method__basis_set=data["basis_set"]))
-        # if data["code_name"] is not None: s.add_calculation_filter(Q("terms", calculations__method__program_name=data["code_name"]))
-        # if data["has_band_structure"] is not None: s.add_calculation_filter(Q("term", calculations__properties__has_electronic_band_structure=data["has_band_structure"]))
-        # if data["has_dos"] is not None: s.add_calculation_filter(Q("term", calculations__properties__has_electronic_dos=data["has_dos"]))
-        # if data["has_thermal_properties"] is not None: s.add_calculation_filter(Q("term", calculations__properties__has_thermodynamical_properties=data["has_thermal_properties"]))
-        # if data["band_gap"] is not None: s.add_calculation_filter(get_range_filter(
-            # "calculations.properties.band_gap",
-            # minimum=data["band_gap"].get("min"),
-            # maximum=data["band_gap"].get("max"),
-            # source_unit=ureg.eV,
-            # target_unit=ureg.J,
-        # ))
-
-        formula = search_by["formula"]
-        elements = search_by["element"]
-        exclusive = search_by["exclusive"]
-
         # The given list of species/formula is reformatted with the Hill system into a
         # query string. With exclusive search we look for exact match, with
         # non-exclusive search we look for match that includes at least all
         # species, possibly even more.
+        formula = search_by["formula"]
+        elements = search_by["elements"]
+        exclusive = search_by["exclusive"]
         if formula is not None:
             element_list = []
             matches = re_formula.finditer(formula)
 
+            prev_end = 0
+            invalid = False
             for match in matches:
+                if match.start() != prev_end:
+                    invalid = True
+                    break
+                prev_end = match.end()
                 groups = match.groups()
                 symbol = groups[0]
                 count = groups[1]
@@ -516,6 +512,14 @@ class EncMaterialsResource(Resource):
                         element_list.append(symbol)
                     else:
                         element_list += [symbol] * int(count)
+            if prev_end != len(formula) or len(element_list) == 0:
+                invalid = True
+            if invalid:
+                abort(400, message=(
+                    "Invalid chemical formula provided. Please use a single "
+                    "continuous string with capitalized element names and "
+                    "integer numbers for the element multiplicity. E.g. 'TiO2'"
+                ))
 
             names, reduced_counts = get_hill_decomposition(element_list, reduced=True)
             query_string = []
@@ -535,7 +539,7 @@ class EncMaterialsResource(Resource):
                     species_and_counts={"query": query_string, "operator": "and"}
                 ))
         elif elements is not None:
-            species, _ = get_hill_decomposition(elements.split(","))
+            species, _ = get_hill_decomposition(elements)
             query_string = " ".join(species)
 
             if exclusive:
@@ -606,8 +610,7 @@ class EncGroupsResource(Resource):
     @api.response(400, "Bad request")
     @api.response(200, "OK", groups_result)
     @api.marshal_with(groups_result)
-    @api.doc("get_material_groups")
-    @api.param("material_id", "28 character identifier for the material.")
+    @api.doc("get_material_groups", params={"material_id": "28 character identifier for the material."})
     @authenticate()
     def get(self, material_id):
         """Returns a summary of the calculation groups that were identified for this material.
@@ -691,10 +694,11 @@ class EncGroupResource(Resource):
     @api.response(400, "Bad request")
     @api.response(200, "OK", group_result)
     @api.marshal_with(group_result)
-    @api.doc("get_material_group")
-    @api.param("group_type", "Type of group. Valid options are: 'eos' and 'par'.")
-    @api.param("group_id", "28 character identifier for the group.")
-    @api.param("material_id", "28 character identifier for the material.")
+    @api.doc("get_material_group", params={
+        "material_id": "28 character identifier for the material.",
+        "group_type": "Type of group. Valid options are: 'eos' and 'par'.",
+        "group_id": "28 character identifier for the group.",
+    })
     @authenticate()
     def get(self, material_id, group_type, group_id):
         """Used to query detailed information about a specific calculation group.
@@ -779,8 +783,8 @@ calculation_result = api.model("calculation_result", {
     "code_version": fields.String,
     "functional_type": fields.String,
     "basis_set_type": fields.String,
-    "core_electron_treatment": fields.String,
-    "run_type": fields.String,
+    "core_electron_treatment": fields.String(default="unavailable"),
+    "run_type": fields.String(default="unavailable"),
     "has_dos": fields.Boolean,
     "has_band_structure": fields.Boolean,
     "has_thermal_properties": fields.Boolean,
@@ -793,7 +797,6 @@ representatives_result = api.model("representatives_result", {
 })
 calculations_result = api.model("calculations_result", {
     "total_results": fields.Integer,
-    "pages": fields.Nested(pages_result),
     "results": fields.List(fields.Nested(calculation_result)),
     "representatives": fields.Nested(representatives_result, skip_none=True),
 })
@@ -805,6 +808,7 @@ class EncCalculationsResource(Resource):
     @api.response(400, "Bad request")
     @api.response(200, "OK", calculations_result)
     @api.doc("get_material_calculations")
+    @api.marshal_with(calculations_result)
     @authenticate()
     def get(self, material_id):
         """Used to return information about all calculations related to the given material.
@@ -943,8 +947,7 @@ class EncStatisticsResource(Resource):
     @api.response(200, "OK", statistics_result)
     @api.expect(statistics_query, validate=False)
     @api.marshal_with(statistics_result, skip_none=True)
-    @api.doc("get_material_statistics")
-    @api.param("material_id", "28 character identifier for the material.")
+    @api.doc("get_material_statistics", params={"material_id": "28 character identifier for the material."})
     @authenticate()
     def post(self, material_id):
         """Used to return statistics related to the specified material and
@@ -1245,52 +1248,55 @@ class EncCalculationResource(Resource):
 
             # Add results from archive
             for key, arch_path in arch_properties.items():
-                value = root[arch_path]
+                try:
+                    value = root[arch_path]
 
-                # Replace unnormalized thermodynamical properties with
-                # normalized ones and turn into dict
-                if key == "thermodynamical_properties":
-                    specific_heat_capacity = value.specific_heat_capacity.magnitude.tolist()
-                    specific_free_energy = value.specific_vibrational_free_energy_at_constant_volume.magnitude.tolist()
-                    specific_heat_capacity = [x if np.isfinite(x) else None for x in specific_heat_capacity]
-                    specific_free_energy = [x if np.isfinite(x) else None for x in specific_free_energy]
-                if isinstance(value, list):
-                    value = [x.m_to_dict() for x in value]
-                else:
-                    value = value.m_to_dict()
-                if key == "thermodynamical_properties":
-                    del value["thermodynamical_property_heat_capacity_C_v"]
-                    del value["vibrational_free_energy_at_constant_volume"]
-                    value["specific_heat_capacity"] = specific_heat_capacity
-                    value["specific_vibrational_free_energy_at_constant_volume"] = specific_free_energy
+                    # Replace unnormalized thermodynamical properties with
+                    # normalized ones and turn into dict
+                    if key == "thermodynamical_properties":
+                        specific_heat_capacity = value.specific_heat_capacity.magnitude.tolist()
+                        specific_free_energy = value.specific_vibrational_free_energy_at_constant_volume.magnitude.tolist()
+                        specific_heat_capacity = [x if np.isfinite(x) else None for x in specific_heat_capacity]
+                        specific_free_energy = [x if np.isfinite(x) else None for x in specific_free_energy]
+                    if isinstance(value, list):
+                        value = [x.m_to_dict() for x in value]
+                    else:
+                        value = value.m_to_dict()
+                    if key == "thermodynamical_properties":
+                        del value["thermodynamical_property_heat_capacity_C_v"]
+                        del value["vibrational_free_energy_at_constant_volume"]
+                        value["specific_heat_capacity"] = specific_heat_capacity
+                        value["specific_vibrational_free_energy_at_constant_volume"] = specific_free_energy
 
-                # DOS results are simplified.
-                if key == "electronic_dos":
-                    if "dos_energies_normalized" in value:
-                        value["dos_energies"] = value["dos_energies_normalized"]
-                        del value["dos_energies_normalized"]
-                    if "dos_values_normalized" in value:
-                        value["dos_values"] = value["dos_values_normalized"]
-                        del value["dos_values_normalized"]
+                    # DOS results are simplified.
+                    if key == "electronic_dos":
+                        if "dos_energies_normalized" in value:
+                            value["dos_energies"] = value["dos_energies_normalized"]
+                            del value["dos_energies_normalized"]
+                        if "dos_values_normalized" in value:
+                            value["dos_values"] = value["dos_values_normalized"]
+                            del value["dos_values_normalized"]
 
-                # Pre-calculate k-path length to be used as x-coordinate in
-                # plots. If the VBM and CBM information is needed later, it
-                # can be added as indices along the path. The exact k-points
-                # and occupations are removed to save some bandwidth.
-                if key == "electronic_band_structure" or key == "phonon_band_structure":
-                    segments = value["section_k_band_segment"]
-                    k_path_length = 0
-                    for segment in segments:
-                        k_points = np.array(segment["band_k_points"])
-                        segment_length = np.linalg.norm(k_points[-1, :] - k_points[0, :])
-                        k_path_distances = k_path_length + np.linalg.norm(k_points - k_points[0, :], axis=1)
-                        k_path_length += segment_length
-                        segment["k_path_distances"] = k_path_distances.tolist()
-                        del segment["band_k_points"]
-                        if "band_occupations" in segment:
-                            del segment["band_occupations"]
+                    # Pre-calculate k-path length to be used as x-coordinate in
+                    # plots. If the VBM and CBM information is needed later, it
+                    # can be added as indices along the path. The exact k-points
+                    # and occupations are removed to save some bandwidth.
+                    if key == "electronic_band_structure" or key == "phonon_band_structure":
+                        segments = value["section_k_band_segment"]
+                        k_path_length = 0
+                        for segment in segments:
+                            k_points = np.array(segment["band_k_points"])
+                            segment_length = np.linalg.norm(k_points[-1, :] - k_points[0, :])
+                            k_path_distances = k_path_length + np.linalg.norm(k_points - k_points[0, :], axis=1)
+                            k_path_length += segment_length
+                            segment["k_path_distances"] = k_path_distances.tolist()
+                            del segment["band_k_points"]
+                            if "band_occupations" in segment:
+                                del segment["band_occupations"]
 
-                result[key] = value
+                    result[key] = value
+                except (AttributeError, KeyError):
+                    abort(500, "Could not find the requested resource.")
 
         # Add results from ES
         for prop, es_source in es_properties.items():
@@ -1399,8 +1405,7 @@ class ReportsResource(Resource):
     @api.response(400, "Bad request")
     @api.response(204, "Report succesfully sent")
     @api.expect(report_query)
-    @api.doc("post_material_report")
-    @api.param("material_id", "28 character identifier for the material.")
+    @api.doc("post_material_report", params={"material_id": "28 character identifier for the material."})
     @authenticate(required=True)
     def post(self, material_id):
         """Post an error report on a material. Requires authentication.
