@@ -1,16 +1,20 @@
-# Copyright 2018 Markus Scheidgen, Alvin Noe Ladines
+#
+# Copyright The NOMAD Authors.
+#
+# This file is part of NOMAD. See https://nomad-lab.eu for further info.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#   http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an"AS IS" BASIS,
+# distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 
 from typing import Iterable, Any, Tuple, Dict, Callable, BinaryIO, Union, List, cast
 from io import BytesIO, BufferedReader
@@ -25,7 +29,7 @@ import re
 from nomad import utils, config, infrastructure
 from nomad.metainfo import Quantity, Reference, Definition, MSection, Section, SubSection
 from nomad.datamodel import EntryArchive
-from nomad.datamodel.metainfo.public import fast_access
+from nomad.datamodel.metainfo.common_dft import FastAccess
 
 '''
 The archive storage is made from two tiers. First the whole archive is stored in
@@ -677,7 +681,7 @@ def filter_archive(
 def create_partial_archive(archive: EntryArchive) -> Dict:
     '''
     Creates a partial archive JSON serializable dict that can be stored directly.
-    The given archive is filtered based on the metainfo category ``fast_access``.
+    The given archive is filtered based on the metainfo category ``FastAccess``.
     Selected sections and other data that they reference (recursively) comprise the
     resulting partial archive.
 
@@ -704,10 +708,10 @@ def create_partial_archive(archive: EntryArchive) -> Dict:
         if section.m_def == EntryArchive.m_def:
             if definition.m_def == Quantity:
                 return True
-            return fast_access.m_def in definition.categories
+            return FastAccess.m_def in definition.categories
 
         if isinstance(definition, Quantity) and isinstance(definition.type, Reference) \
-                and fast_access.m_def in definition.categories:
+                and FastAccess.m_def in definition.categories:
             # Reference list in partial archives are not supported
             if definition.is_scalar:
                 referenced = getattr(section, definition.name)
@@ -715,7 +719,7 @@ def create_partial_archive(archive: EntryArchive) -> Dict:
                     referenceds.append(referenced)
 
         if isinstance(definition, SubSection):
-            return fast_access.m_def in definition.categories
+            return FastAccess.m_def in definition.categories
 
         return True
 
@@ -893,13 +897,13 @@ def compute_required_with_referenced(required):
             prop = key.split('[')[0]
             prop_definition = parent.all_properties[prop]
             if isinstance(prop_definition, SubSection):
-                if fast_access.m_def not in prop_definition.categories:
+                if FastAccess.m_def not in prop_definition.categories:
                     raise _Incomplete()
 
                 traverse(value, prop_definition.sub_section)
             if isinstance(prop_definition, Quantity) and isinstance(prop_definition.type, Reference):
                 current[prop] = '*'
-                if fast_access.m_def not in prop_definition.categories:
+                if FastAccess.m_def not in prop_definition.categories:
                     continue
 
                 target_section_def = prop_definition.type.target_section_def.m_resolved()
