@@ -1,16 +1,20 @@
-# Copyright 2018 Markus Scheidgen, empty_task
+#
+# Copyright The NOMAD Authors.
+#
+# This file is part of NOMAD. See https://nomad-lab.eu for further info.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#   http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an"AS IS" BASIS,
+# distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 
 '''
 This module describes all configurable parameters for the nomad python code. The
@@ -152,12 +156,18 @@ services = NomadConfig(
     not_processed_value='not processed',
     unavailable_value='unavailable',
     https=False,
+    https_upload=False,
     upload_limit=10,
     force_raw_file_decoding=False,
     download_scan_size=500,
     download_scan_timeout=u'30m'
 )
 
+oasis = NomadConfig(
+    central_nomad_api_url='https://nomad-lab.eu/prod/rae/api',
+    central_nomad_deployment_id='nomad-lab.eu/prod/rae'
+)
+
 tests = NomadConfig(
     default_timeout=30
 )
@@ -200,7 +210,7 @@ def check_config():
             "--matches-only'."
         )
 
-    if not os.path.exists(normalize.springer_db_path):
+    if normalize.springer_db_path and not os.path.exists(normalize.springer_db_path):
         normalize.springer_db_path = None
 
 
@@ -271,7 +281,7 @@ datacite = NomadConfig(
 )
 
 meta = NomadConfig(
-    version='0.9.5',
+    version='0.9.8',
     commit=gitinfo.commit,
     release='devel',
     default_domain='dft',
@@ -280,11 +290,16 @@ meta = NomadConfig(
     description='A FAIR data sharing platform for materials science data',
     homepage='https://nomad-lab.eu',
     source_url='https://gitlab.mpcdf.mpg.de/nomad-lab/nomad-FAIR',
-    maintainer_email='markus.scheidgen@physik.hu-berlin.de'
+    maintainer_email='markus.scheidgen@physik.hu-berlin.de',
+    deployment_id='nomad-lab.eu/prod/rae'
+)
+
+gitlab = NomadConfig(
+    private_token='not set'
 )
 
 auxfile_cutoff = 100
-parser_matching_size = 9128
+parser_matching_size = 150 * 80  # 150 lines of 80 ASCII characters per line
 console_log_level = logging.WARNING
 max_upload_size = 32 * (1024 ** 3)
 raw_file_strip_cutoff = 1000
