@@ -1,16 +1,20 @@
-# Copyright 2018 Markus Scheidgen
+#
+# Copyright The NOMAD Authors.
+#
+# This file is part of NOMAD. See https://nomad-lab.eu for further info.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#   http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an"AS IS" BASIS,
+# distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 
 ''' All generic entry metadata and related classes. '''
 
@@ -23,7 +27,7 @@ from nomad import metainfo, config
 from nomad.metainfo.search_extension import Search
 from nomad.metainfo.elastic_extension import ElasticDocument
 from nomad.metainfo.mongoengine_extension import Mongo, MongoDocument
-from nomad.datamodel.metainfo.public import fast_access
+from nomad.datamodel.metainfo.common_dft import FastAccess
 
 from .dft import DFTMetadata
 from .ems import EMSMetadata
@@ -34,9 +38,9 @@ from .qcms import QCMSMetadata
 m_package = metainfo.Package()
 
 from .encyclopedia import EncyclopediaMetadata  # noqa
-from .metainfo.public import section_run, Workflow  # noqa
+from .metainfo.common_dft import Run, Workflow  # noqa
 from .metainfo.common_experimental import Experiment  # noqa
-from .metainfo.general_qcms import QuantumCMS  # noqa
+from .metainfo.common_qcms import QuantumCMS  # noqa
 
 
 def _only_atoms(atoms):
@@ -102,6 +106,8 @@ class User(Author):
     is_admin = metainfo.Quantity(
         type=bool, derived=lambda user: user.user_id == config.services.admin_user_id)
 
+    is_oasis_admin = metainfo.Quantity(type=bool, default=False)
+
     @staticmethod
     @cached(cache=TTLCache(maxsize=2048, ttl=24 * 3600))
     def get(*args, **kwargs) -> 'User':
@@ -235,6 +241,11 @@ class EditableUserMetadata(metainfo.MCategory):
     m_def = metainfo.Category(categories=[UserProvidableMetadata])
 
 
+class OasisMetadata(metainfo.MCategory):
+    ''' NOMAD entry metadata quantities that can be provided by an OASIS. '''
+    m_def = metainfo.Category(categories=[EditableUserMetadata])
+
+
 class MongoMetadata(metainfo.MCategory):
     ''' NOMAD entry quantities that are stored in mongodb and not necessarely in the archive. '''
     pass
@@ -307,6 +318,7 @@ class EntryMetadata(metainfo.MSection):
     calc_id = metainfo.Quantity(
         type=str,
         description='A persistent and globally unique identifier for the entry',
+        categories=[OasisMetadata],
         a_search=Search(many_or='append'))
 
     calc_hash = metainfo.Quantity(
@@ -371,7 +383,7 @@ class EntryMetadata(metainfo.MSection):
     published = metainfo.Quantity(
         type=bool, default=False,
         description='Indicates if the entry is published',
-        categories=[MongoMetadata],
+        categories=[MongoMetadata, OasisMetadata],
         a_search=Search())
 
     processed = metainfo.Quantity(
@@ -474,7 +486,7 @@ class EntryMetadata(metainfo.MSection):
         a_search=Search())
 
     upload_time = metainfo.Quantity(
-        type=metainfo.Datetime, categories=[MongoMetadata],
+        type=metainfo.Datetime, categories=[MongoMetadata, OasisMetadata],
         description='The date and time this entry was uploaded to nomad',
         a_flask=dict(admin_only=True),
         a_search=Search(order_default=True))
@@ -507,7 +519,7 @@ class EntryMetadata(metainfo.MSection):
         a_search=Search(many_or='split'))
 
     last_edit = metainfo.Quantity(
-        type=metainfo.Datetime, categories=[MongoMetadata],
+        type=metainfo.Datetime, categories=[MongoMetadata, OasisMetadata],
         description='The date and time the user metadata was edited last',
         a_search=Search())
 
@@ -534,9 +546,9 @@ class EntryMetadata(metainfo.MSection):
         a_search=Search())
 
     ems = metainfo.SubSection(sub_section=EMSMetadata, a_search='ems')
-    dft = metainfo.SubSection(sub_section=DFTMetadata, a_search='dft', categories=[fast_access])
+    dft = metainfo.SubSection(sub_section=DFTMetadata, a_search='dft', categories=[FastAccess])
     qcms = metainfo.SubSection(sub_section=QCMSMetadata, a_search='qcms')
-    encyclopedia = metainfo.SubSection(sub_section=EncyclopediaMetadata, categories=[fast_access], a_search='encyclopedia')
+    encyclopedia = metainfo.SubSection(sub_section=EncyclopediaMetadata, categories=[FastAccess], a_search='encyclopedia')
 
     def apply_user_metadata(self, metadata: dict):
         ''' Applies a user provided metadata dict to this calc. '''
@@ -559,11 +571,11 @@ class EntryMetadata(metainfo.MSection):
 
 class EntryArchive(metainfo.MSection):
 
-    section_run = metainfo.SubSection(sub_section=section_run, repeats=True)
+    section_run = metainfo.SubSection(sub_section=Run, repeats=True)
     section_experiment = metainfo.SubSection(sub_section=Experiment)
     section_quantum_cms = metainfo.SubSection(sub_section=QuantumCMS)
-    section_workflow = metainfo.SubSection(sub_section=Workflow, categories=[fast_access])
-    section_metadata = metainfo.SubSection(sub_section=EntryMetadata, categories=[fast_access])
+    section_workflow = metainfo.SubSection(sub_section=Workflow, categories=[FastAccess])
+    section_metadata = metainfo.SubSection(sub_section=EntryMetadata, categories=[FastAccess])
 
     processing_logs = metainfo.Quantity(
         type=Any, shape=['0..*'],
