@@ -1,16 +1,20 @@
-# Copyright 2018 Markus Scheidgen
+#
+# Copyright The NOMAD Authors.
+#
+# This file is part of NOMAD. See https://nomad-lab.eu for further info.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#   http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an"AS IS" BASIS,
+# distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 
 '''
 DFT specific metadata
@@ -23,8 +27,8 @@ from nomad.metainfo import MSection, Section, Quantity, MEnum, SubSection
 from nomad.metainfo.search_extension import Search
 
 from .optimade import OptimadeEntry
-from .metainfo.public import Workflow, fast_access
-from .metainfo.public import section_XC_functionals
+from .metainfo.common_dft import Workflow, FastAccess
+from .metainfo.common_dft import section_XC_functionals
 
 
 xc_treatments = {
@@ -234,7 +238,7 @@ class DFTMetadata(MSection):
         a_search=Search(many_or='append', group='groups_grouped', metric_name='groups', metric='cardinality'))
 
     labels = SubSection(
-        sub_section=Label, repeats=True, categories=[fast_access],
+        sub_section=Label, repeats=True, categories=[FastAccess],
         description='The labels taken from AFLOW prototypes and springer.',
         a_search='labels')
 
