@@ -1,16 +1,20 @@
-# Copyright 2018 Markus Scheidgen
+#
+# Copyright The NOMAD Authors.
+#
+# This file is part of NOMAD. See https://nomad-lab.eu for further info.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#   http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an"AS IS" BASIS,
+# distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 
 '''
 Uploads contains classes and functions to create and maintain file structures
@@ -58,6 +62,7 @@ import tarfile
 import hashlib
 import io
 import pickle
+import json
 
 from nomad import config, utils, datamodel
 from nomad.archive import write_archive, read_archive, ArchiveReader
@@ -427,6 +432,7 @@ class StagingUploadFiles(UploadFiles):
         Replaces the staging upload data with a public upload record by packing all
         data into files. It is only available if upload *is_bag*.
         This is potentially a long running operation.
+
         Arguments:
             upload: The upload with all calcs and  calculation metadata of the upload
                 used to determine what files to pack and what the embargo situation is.
@@ -702,7 +708,9 @@ class PublicUploadFilesBasedStagingUploadFiles(StagingUploadFiles):
         assert False, 'do not add_rawfiles to a %s' % self.__class__.__name__
 
     def pack(self, entries: Iterable[datamodel.EntryMetadata], *args, **kwargs) -> None:
-        ''' Packs only the archive contents and stores it in the existing public upload files. '''
+        '''
+        Packs only the archive contents and stores it in the existing public upload files.
+        '''
         super().pack(entries, target_dir=self.public_upload_files, skip_raw=True)
 
 
@@ -798,6 +806,16 @@ class PublicUploadFiles(UploadFiles):
 
         return staging_upload_files
 
+    def add_metadata_file(self, metadata: dict):
+        zip_path = self._zip_file_object('raw', 'public', 'plain').os_path
+        with zipfile.ZipFile(zip_path, 'a') as zf:
+            with zf.open('nomad.json', 'w') as f:
+                f.write(json.dumps(metadata).encode())
+
+    @property
+    def public_raw_data_file(self):
+        return self._zip_file_object('raw', 'public', 'plain').os_path
+
     def raw_file(self, file_path: str, *args, **kwargs) -> IO:
         return self._file_in_zip('raw', 'plain', file_path, *args, *kwargs)
 
