@@ -1,16 +1,20 @@
-# Copyright 2018 Markus Scheidgen
+#
+# Copyright The NOMAD Authors.
+#
+# This file is part of NOMAD. See https://nomad-lab.eu for further info.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#   http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an"AS IS" BASIS,
+# distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 
 '''
 This module provides function to establish connections to the database, searchengine, etc.
@@ -332,12 +336,14 @@ class Keycloak():
         from nomad import datamodel
 
         kwargs = {key: value[0] for key, value in keycloak_user.get('attributes', {}).items()}
+        oasis_admin = kwargs.pop('is_oasis_admin', None) is not None
         return datamodel.User(
             user_id=keycloak_user['id'],
             email=keycloak_user.get('email'),
             username=keycloak_user.get('username'),
             first_name=keycloak_user.get('firstName'),
             last_name=keycloak_user.get('lastName'),
+            is_oasis_admin=oasis_admin,
             created=datetime.fromtimestamp(keycloak_user['createdTimestamp'] / 1000),
             **kwargs)
 
@@ -491,6 +497,7 @@ def send_mail(name: str, email: str, message: str, subject: str):
     msg = MIMEText(message)
     msg['Subject'] = subject
     msg['To'] = name
+    msg['From'] = config.mail.from_address
     to_addrs = [email]
 
     if config.mail.cc_address is not None:
