@@ -1,16 +1,20 @@
-# Copyright 2018 Markus Scheidgen
+#
+# Copyright The NOMAD Authors.
+#
+# This file is part of NOMAD. See https://nomad-lab.eu for further info.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#   http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an"AS IS" BASIS,
+# distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 
 import numpy as np
 import ase
@@ -87,6 +91,9 @@ class BandStructureNormalizer(Normalizer):
         (https://atztogo.github.io/spglib/definition.html#transformation-to-the-primitive-cell)
         whether the k-point path stays inside the first Brillouin zone.
         """
+        # If reciprocal cell is reported by parser, use it.
+        if band.reciprocal_cell is not None:
+            return
         try:
             orig_atoms = system.m_cache["representative_atoms"]
             symmetry_analyzer = system.section_symmetry[0].m_cache["symmetry_analyzer"]
@@ -306,7 +313,7 @@ class BandStructureNormalizer(Normalizer):
         """Return dict of special points.
 
         The definitions are from a paper by Wahyu Setyawana and Stefano
-        Curtarolo::
+        Curtarolo:
 
             http://dx.doi.org/10.1016/j.commatsci.2010.05.010
 
