@@ -1,16 +1,20 @@
-# Copyright 2018 Markus Scheidgen
 #
-# Licensed under the Apache License, Version 2.0 (the 'License');
+# Copyright The NOMAD Authors.
+#
+# This file is part of NOMAD. See https://nomad-lab.eu for further info.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#   http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an'AS IS' BASIS,
+# distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 
 from typing import Dict, List
 from math import gcd, isnan
@@ -128,7 +132,7 @@ class MaterialBulkNormalizer(MaterialNormalizer):
 
     def lattice_parameters(self, ideal: IdealizedStructure, std_atoms: Atoms) -> None:
         cell_normalized = std_atoms.get_cell() * 1E-10
-        param_values = atomutils.get_lattice_parameters(cell_normalized)
+        param_values = atomutils.cell_to_cellpar(cell_normalized)
         param_section = ideal.m_create(LatticeParameters)
         param_section.a = float(param_values[0])
         param_section.b = float(param_values[1])
