@@ -1,23 +1,29 @@
-# Copyright 2018 Markus Scheidgen
+#
+# Copyright The NOMAD Authors.
+#
+# This file is part of NOMAD. See https://nomad-lab.eu for further info.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#   http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an"AS IS" BASIS,
+# distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 
 from typing import List
 from abc import ABCMeta, abstractmethod
 import re
+import os
+import os.path
+from functools import lru_cache
 
 from nomad import config
-from nomad.metainfo import Environment
 from nomad.datamodel import EntryArchive, UserProvidableMetadata, EntryMetadata
 
 
@@ -30,11 +36,6 @@ class Parser(metaclass=ABCMeta):
 
     def __init__(self):
         self.domain = 'dft'
-        self._metainfo_env: Environment = None
-
-    @property
-    def metainfo_env(self):
-        return self._metainfo_env
 
     @abstractmethod
     def is_mainfile(
@@ -107,6 +108,8 @@ class MatchingParser(Parser):
         mainfile_contents_re: A regexp that is used to match the first 1024 bytes of a
             potential mainfile.
         mainfile_name_re: A regexp that is used to match the paths of potential mainfiles
+        mainfile_alternative: If True files are mainfile if no mainfile_name_re matching file
+            is present in the same directory.
         domain: The domain that this parser should be used for. Default is 'dft'.
         supported_compressions: A list of [gz, bz2], if the parser supports compressed files
     '''
@@ -116,6 +119,7 @@ class MatchingParser(Parser):
             mainfile_binary_header: bytes = None,
             mainfile_mime_re: str = r'text/.*',
             mainfile_name_re: str = r'.*',
+            mainfile_alternative: bool = False,
             domain='dft',
             supported_compressions: List[str] = []) -> None:
 
@@ -127,6 +131,7 @@ class MatchingParser(Parser):
         self._mainfile_binary_header = mainfile_binary_header
         self._mainfile_mime_re = re.compile(mainfile_mime_re)
         self._mainfile_name_re = re.compile(mainfile_name_re)
+        self._mainfile_alternative = mainfile_alternative
         # Assign private variable this way to avoid static check issue.
         if mainfile_contents_re is not None:
             self._mainfile_contents_re = re.compile(mainfile_contents_re)
@@ -134,6 +139,8 @@ class MatchingParser(Parser):
             self._mainfile_contents_re = None
         self._supported_compressions = supported_compressions
 
+        self._ls = lru_cache(maxsize=16)(lambda directory: os.listdir(directory))
+
     def is_mainfile(
             self, filename: str, mime: str, buffer: bytes, decoded_buffer: str,
             compression: str = None) -> bool:
@@ -141,15 +148,34 @@ class MatchingParser(Parser):
         if self._mainfile_binary_header is not None:
             if self._mainfile_binary_header not in buffer:
                 return False
+
         if self._mainfile_contents_re is not None:
             if decoded_buffer is not None:
                 if self._mainfile_contents_re.search(decoded_buffer) is None:
                     return False
             else:
                 return False
-        return self._mainfile_mime_re.match(mime) is not None and \
-            self._mainfile_name_re.fullmatch(filename) is not None and \
-            (compression is None or compression in self._supported_compressions)
+
+        if self._mainfile_mime_re.match(mime) is None:
+            return False
+
+        if compression is not None and compression not in self._supported_compressions:
+            return False
+
+        if self._mainfile_name_re.fullmatch(filename) is None:
+            if not self._mainfile_alternative:
+                return False
+
+            directory = os.path.dirname(filename)
+            for sibling in self._ls(directory):
+                sibling = os.path.join(directory, sibling)
+                sibling_is_mainfile = sibling != filename and \
+                    self._mainfile_name_re.fullmatch(sibling) is not None and \
+                    os.path.isfile(sibling)
+                if sibling_is_mainfile:
+                    return False
+
+        return True
 
     def __repr__(self):
         return self.name
