@@ -1,16 +1,20 @@
-# Copyright 2018 Markus Scheidgen
+#
+# Copyright The NOMAD Authors.
+#
+# This file is part of NOMAD. See https://nomad-lab.eu for further info.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#   http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an"AS IS" BASIS,
+# distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 
 from typing import List, Any, Dict
 import logging
@@ -163,6 +167,7 @@ class Proc(Document, metaclass=ProcMetaclass):
 
     errors = ListField(StringField())
     warnings = ListField(StringField())
+    last_status_message = StringField(default=None)
 
     current_process = StringField(default=None)
     process_status = StringField(default=None)
@@ -286,6 +291,8 @@ class Proc(Document, metaclass=ProcMetaclass):
         self.on_fail()
 
         logger.info('process failed')
+        if len(self.errors) > 0:
+            self.last_satus_message = 'ERROR: %s' % self.errors[-1]
 
         self.save()
 
